@@ -60,6 +60,7 @@ class Option:
                  default: ty.Any = OMITTED,
                  default_factory: ty.Callable = OMITTED,
                  default_by_run=OMITTED,
+                 child_option: bool = False,
                  track: bool = True,
                  help: str = ''):
         """
@@ -71,6 +72,9 @@ class Option:
          - Callable. Will be called with run_id, must return value for run.
          - List [(start_run_id, value), ..,] for values specified by range of
            runs.
+        :param child_option: If true option is marked as a child_option. In case of
+            plugin which is inherited from an already existing plugin we only store the
+            child options in the lineage of the plugin.
         :param track: If True (default), option value becomes part of plugin
         lineage (just like the plugin version).
         :param help: Human-readable description of the option.
@@ -82,6 +86,7 @@ class Option:
         self.default_factory = default_factory
         self.track = track
         self.help = help
+        self.child_option = child_option
 
         # if self.default_by_run is not OMITTED:
         #     warnings.warn(f"The {self.name} option uses default_by_run,"
