@@ -37,7 +37,6 @@ def baseline(records, baseline_samples=40, flip=True,
     """
     if not len(records):
         return records
-    samples_per_record = len(records[0]['data'])
 
     # Array for looking up last baseline (mean, rms) seen in channel
     # We only care about the channels in this set of records; a single .max()
@@ -66,7 +65,7 @@ def baseline(records, baseline_samples=40, flip=True,
         # Subtract baseline from all data samples in the record
         # (any additional zeros should be kept at zero)
         d['data'][:d['length']] = (
-            (-1 * flip) * (d['data'][:d['length']] - int(bl)))
+            (-1 if flip else 1) * (d['data'][:d['length']] - int(bl)))
         d['baseline'] = bl
         d['baseline_rms'] = rms
 
@@ -125,7 +124,7 @@ def integrate(records):
         return
     for i, r in enumerate(records):
         records[i]['area'] = (
-            r['data'].sum()
+            r['data'].sum() * 2**r['amplitude_bit_shift']
             # Add floating part of baseline * number of samples
             # int(round()) the result since the area field is an int
             + int(round((r['baseline'] % 1) * r['length'])))
@@ -192,6 +191,8 @@ def find_hits(records,
 
     NB: returned hits are NOT sorted yet!
     """
+    if not len(records):
+        return np.zeros(0, dtype=strax.hit_dtype)
     if isinstance(min_amplitude, (tuple, list)):
         min_amplitude = np.array(min_amplitude)
     if isinstance(min_height_over_noise, (tuple, list)):
@@ -226,8 +227,10 @@ def find_hits(records,
 # else copying buffers / switching context dominates over actual computation
 # No max_duration argument: hits terminate at record boundaries, and
 # anyone insane enough to try O(sec) long records deserves to be punished
+# TODO: this ignores amplitude_bit_shift, since this function also
+# has to support raw_records, which don't have that field.
 @strax.growing_result(strax.hit_dtype, chunk_size=int(1e4))
-@numba.jit(nopython=True, nogil=True, cache=True)
+@numba.njit(nogil=True, cache=True)
 def _find_hits(records, min_amplitude, min_height_over_noise,
                _result_buffer=None):
     buffer = _result_buffer
@@ -257,7 +260,7 @@ def _find_hits(records, min_amplitude, min_height_over_noise,
         for i in range(n_samples):
             # We can't use enumerate over r['data'],
             # numba gives errors if we do.
-            # TODO: file issue?
+            # maybe file an issue?
             x = r['data'][i]
 
             satisfy_threshold = x >= threshold
